import { I18nextConfig } from '@genesislcap/foundation-i18n';
import { ComposableStyles, PartialFASTElementDefinition } from '@microsoft/fast-element';
import type { AuthRoutes } from '../routes/types';
import { TemplateOptions } from './templates';
/**
 * Do not reference any ../main files here. We must avoid ../tags being referenced and therefore setup pre-configuration.
 */
/**
 * @public
 * Could be expanded to allow field ViewTemplates.
 */
export type FieldConfig = {
    label: string;
    /**
     * @privateRemarks
     * Added for now to apply defaults, but I'm not convinced we should do this. Could be made reactive after upgrade.
     */
    value?: any;
    /**
     * @remarks
     * We expect fields to be required by default, but you can override to make them optional. Organisation for example.
     */
    required?: boolean;
    /**
     * @remarks
     * A regular expression the control's value should match. Please note the email field has its own built in pattern.
     */
    pattern?: string;
    /**
     * @remarks
     * Used in conjunction with the pattern to explain the conditions.
     */
    title?: string;
};
/**
 * @public
 * @privateRemarks
 * Using null to exclude the organisation field rather than boolean, as it affords cleaner value lookups in the templates.
 */
export type FieldConfigMap = {
    organisation: FieldConfig | null;
    username: FieldConfig;
    email: FieldConfig;
    password: FieldConfig;
};
/**
 * @public
 */
export type SSOConfig = {
    /**
     * Optionally toggle the SSO journey on/off by default.
     */
    toggled?: boolean;
};
/**
 * AuthConfig DI interface.
 *
 * @public
 */
export interface AuthConfig extends PartialFASTElementDefinition {
    /**
     * Template options.
     *
     * @remarks
     * Used by host applications to assign MF template options and subcomponent tags to align with the host design system.
     */
    templateOptions: TemplateOptions;
    /**
     * Logo styles.
     *
     * @remarks
     * Providing `null` will hide the logo. Use content: url(...); to set source.
     *
     * @example Setting a custom logo using a plain template literal
     * ```ts
     * logo: `
     *     content: url(${customLogoImport});
     * `,
     * ```
     *
     * @example Setting a custom logo using FASTElement's `css` helper or a pre-created ElementStyles instance when your host app is FAST based
     * ```ts
     * logo: css`
     *     content: url(${customLogoImport});
     * `,
     * ...
     * logo: LogoElementStyles,
     * ```
     */
    logo: ComposableStyles | ComposableStyles[] | null;
    /**
     * Background styles.
     *
     * @remarks
     * Providing `null` will hide the background. The format examples from {@link (AuthConfig:interface).logo} apply here too.
     * See also {@link https://developer.mozilla.org/en-US/docs/Web/CSS/background} for more information.
     *
     * @example Setting a custom background image
     * ```ts
     * background: `
     *     :host {
     *         background-image: url(${customBGImport});
     *     }
     * `,
     * ```
     */
    background: ComposableStyles | ComposableStyles[] | null;
    /**
     * Background video.
     *
     * @remarks
     * Overlays any background image if set. Defaults to `null`.
     */
    backgroundVideo: {
        src: string;
        type: 'video/ogg' | 'video/mp4' | 'video/webm';
    } | null;
    /**
     * Omit any of the internal routes except 'login' and 'not-found'.
     *
     * @remarks
     * Routes specified here will not be configured nor will any internal cross-links to them be displayed.
     *
     * @example
     * ```ts
     * omitRoutes: ['request-account'],
     * ```
     */
    omitRoutes: Exclude<AuthRoutes, 'login' | 'not-found'>[];
    /**
     * Map of configuration for each of the primary form fields.
     *
     * @remarks
     * Use to drive inclusion / exclusion, tailor labels etc.
     */
    fields: FieldConfigMap;
    /**
     * The `path` of the micro frontend as defined in the parent / host route.
     *
     * @remarks
     * Used to help resolve internal route links. Defaults to an empty string.
     */
    hostPath: string;
    /**
     * Omit certain return URLs which the session service may have captured.
     *
     * @example
     * ```ts
     * omitRedirectUrls: ['/not-found'],
     * ```
     */
    omitRedirectUrls: string[];
    /**
     * @remarks
     * Called after a successful login and should be overridden by host. User.lastPath() can be used to trump static
     * values when required.
     */
    postLoginRedirect: () => void;
    /**
     * @remarks
     * Called after a successful logout. Routes to back to the login form by default.
     */
    postLogoutRedirect: () => void;
    /**
     * SSO configuration.
     *
     * @remarks
     * Providing `null` excludes the SSO journey option completely.
     */
    sso: SSOConfig | null;
    /**
     * Display arbitrary version information.
     *
     * @remarks
     * Applications may choose to set this to display their Application release version on the login form.
     *
     * @example
     * ```ts
     * versionInformation: 'Version 3.0.0',
     * ```
     */
    versionInformation?: string;
    /**
     * Feedback delay.
     *
     * @remarks
     * The amount of seconds to display user feedback messages, such as "check your inbox" etc. Defaults to 3 seconds.
     */
    feedbackDelay?: number;
    /**
     * I18n resources.
     *
     * @remarks
     * This property holds the localization resources needed for internationalization of the application.
     * It should follow the structure defined by the I18nextConfig interface, which includes
     * translations grouped by locale and namespace. Each locale can contain multiple namespaces, and each
     * namespace includes key-value pairs for translation strings.
     *
     * @example
     * ```ts
     * resources: {
     *   en: {
     *     'foundation-auth': {
     *       key: 'Hello World'
     *     }
     *   },
     *   es: {
     *     'foundation-auth': {
     *       key: 'Hola Mundo'
     *     }
     *   }
     * }
     * ```
     */
    localizationResources?: I18nextConfig['resources'];
}
/**
 * Default AuthConfig DI implementation.
 * @public
 */
export declare const defaultAuthConfig: AuthConfig;
/**
 * AuthConfig DI key.
 *
 * @internal
 * @privateRemarks
 * Marked as internal to stop api-extractor becoming confused cross-linking tokens with the same name.
 */
export declare const AuthConfig: import("@microsoft/fast-foundation").InterfaceSymbol<AuthConfig>;
//# sourceMappingURL=config.d.ts.map