import { AbstractMachine, FetchMachine, FetchMachineEvent, Machine } from '@genesislcap/foundation-state-machine';
import { User } from '@genesislcap/foundation-user';
import { UUID } from '@genesislcap/foundation-utils';
import type { InterpreterOptions } from 'xstate';
import { AuthConfig } from '../config/config';
import { CredentialManager } from '../credential';
import { AuthMessageMapper } from '../mapper';
import { AuthRouting } from '../routes/routing';
import type { AuthMachineContext, AuthMachineEvent, LoginParams, QueryParams } from './types';
/**
 * AuthMachine interface.
 * @public
 */
export interface AuthMachine extends Machine<AuthMachineContext, AuthMachineEvent> {
}
/**
 * Default AuthMachine.
 * @public
 */
export declare class DefaultAuthMachine extends AbstractMachine<AuthMachineContext, AuthMachineEvent> implements AuthMachine {
    private user;
    private fetchMachine;
    private cm;
    private uuid;
    private messageMapper;
    private config;
    private routing;
    constructor(user: User, fetchMachine: FetchMachine, cm: CredentialManager, uuid: UUID, messageMapper: AuthMessageMapper, config: AuthConfig, routing: AuthRouting);
    start(options?: InterpreterOptions<typeof this.machine>): void;
    /**
     * AuthMachine is a singleton.
     */
    getFromContainer(): this;
    protected captureAuthQueryParams(): QueryParams;
    protected convertQueryParamsToLoginParams(params: QueryParams): LoginParams;
    protected createFetchMachineEvent(url: string, msg?: {}): FetchMachineEvent;
    protected onIdleEvents: () => {
        login: {
            actions: string[];
            target: string;
        };
        "forgot-password": {
            actions: string[];
            target: string;
        };
        "change-password": {
            actions: string[];
            target: string;
        };
        "reset-password": {
            actions: string[];
            target: string;
        };
    };
    protected dtoTransition: () => {
        actions: string[];
        target: string;
    };
}
/**
 * AuthMachine DI key.
 * @internal
 */
export declare const AuthMachine: import("@microsoft/fast-foundation").InterfaceSymbol<AuthMachine>;
/**
 * Get AuthMachine from the DI container.
 *
 * @remarks
 * A utility method for host applications that are not using decorators or the DI container.
 *
 * @example
 * ```ts
 * import { getAuthMachine } from '@genesislcap/foundation-auth/machine';
 * import { getUser, UserAuthenticationChangeEvent } from '@genesislcap/foundation-user';
 * ...
 * private authMachine = getAuthMachine();
 * ...
 * private user = getUser();
 * private unsubscribeUser = this.user.addAuthenticationChangeListener((event: UserAuthenticationChangeEvent) => {
 *   console.log('User authenticated?', event.detail); // < boolean === this.user.isAuthenticated
 * });
 * ...
 * this.authMachine.send({
 *   type: 'login',
 *   params: {
 *     type: CredentialType.BASIC,
 *     username: 'JohnWick',
 *     password: '***********',
 *   },
 * });
 * ```
 *
 * @public
 */
export declare function getAuthMachine(): AuthMachine;
//# sourceMappingURL=machine.d.ts.map