import { MESSAGE_TYPE } from '../dto';
import type { LoginDTO, LoginEntity, LoginMapper } from './loginMapper';
import type { LogoutDTO, LogoutEntity, LogoutMapper } from './logoutMapper';
import type { PasswordManagementDTO, PasswordManagementEntity, PasswordManagementMapper } from './passwordManagementMapper';
/**
 * DTO mapper interface.
 * @remarks
 * Both `toEntity` and `toDTO` APIs are optional by design as message direction flows are unknown.
 * @typeParam TDTO - The dto type.
 * @typeParam TEntity - The entity type.
 * @public
 */
export interface Mapper<TDTO, TEntity> {
    /**
     * Converts a DTO to an entity.
     * @param dto - The DTO to convert.
     * @returns The resulting entity.
     */
    toEntity?: (dto: TDTO) => TEntity;
    /**
     * Converts an entity to a DTO.
     * @param entity - The entity to convert.
     * @returns The resulting DTO.
     */
    toDTO?: (entity: Partial<TEntity>) => TDTO;
}
/**
 * @public
 */
export type AuthDTO = LoginDTO | LogoutDTO | PasswordManagementDTO;
/**
 * @public
 */
export type AuthEntity = LoginEntity | LogoutEntity | PasswordManagementEntity;
/**
 * @public
 */
export type AuthMessageMapperLookup = {
    /**
     * Login
     */
    [MESSAGE_TYPE.LOGIN]: LoginMapper;
    [MESSAGE_TYPE.LOGIN_ACK]: LoginMapper;
    [MESSAGE_TYPE.LOGIN_NACK]: LoginMapper;
    [MESSAGE_TYPE.LOGIN_DETAILS_ACK]: LoginMapper;
    [MESSAGE_TYPE.LOGIN_DETAILS_NACK]: LoginMapper;
    /**
     * Logout
     */
    [MESSAGE_TYPE.LOGOUT_ACK]: LogoutMapper;
    [MESSAGE_TYPE.LOGOUT_NACK]: LogoutMapper;
    /**
     * Password management
     */
    [MESSAGE_TYPE.FORGOT_PASSWORD]: PasswordManagementMapper;
    [MESSAGE_TYPE.CHANGE_PASSWORD]: PasswordManagementMapper;
    [MESSAGE_TYPE.RESET_PASSWORD]: PasswordManagementMapper;
    /**
     * Generic (shouldn't really get these)
     * TODO: Update to use a generic mapper, for now let logout mapper surface error
     */
    [MESSAGE_TYPE.EVENT_NACK]: LogoutMapper;
};
//# sourceMappingURL=types.d.ts.map