import { CustomEventMap } from '@genesislcap/foundation-events';
import { AbstractStoreRoot, StoreRoot, StoreRootEventDetailMap } from '@genesislcap/foundation-store';
import { AuthConfig } from '../config/config';
import { AuthMachine } from '../machine/machine';
import type { AuthMachineEventMap, ChangePasswordParams, ForgotPasswordParams, LoginParams, ResetPasswordParams } from '../machine/types';
import { AuthFormEntryStore } from './authFormEntry';
/**
 * @public
 */
export type AuthEventMap = StoreRootEventDetailMap & AuthMachineEventMap & {
    'auth-login': LoginParams;
    'auth-logout': void;
    'auth-change-password': ChangePasswordParams;
    'auth-forgot-password': ForgotPasswordParams;
    'auth-reset-password': ResetPasswordParams;
};
declare global {
    interface HTMLElementEventMap extends CustomEventMap<AuthEventMap> {
    }
}
type InternalAuthEventMap = {};
/**
 * AuthStore interface.
 * @remarks
 * This is the root store for the foundation-auth package.
 * @public
 */
export interface AuthStore extends StoreRoot {
    readonly isPending: boolean;
    readonly formEntry: AuthFormEntryStore;
    readonly metaMessages: string[];
    readonly errorMessages: string[];
    /**
     * Auth workflows and transient state (context in xstate lingo).
     */
    readonly authMachine: AuthMachine;
    /**
     * @param event - The machine mediation event.
     */
    onMachineEvent(event: CustomEvent<AuthEventMap['auth-machine-event']>): void;
    onLogin(event: CustomEvent<LoginParams>): void;
    onLogout(event: CustomEvent<void>): void;
    onChangePassword(event: CustomEvent<ChangePasswordParams>): void;
    onForgotPassword(event: CustomEvent<ForgotPasswordParams>): void;
    onResetPassword(event: CustomEvent<ResetPasswordParams>): void;
}
/**
 * Default AuthStore.
 * @public
 */
export declare class DefaultAuthStore extends AbstractStoreRoot<AuthStore, AuthEventMap, InternalAuthEventMap> implements AuthStore {
    readonly config: AuthConfig;
    readonly authMachine: AuthMachine;
    readonly formEntry: AuthFormEntryStore;
    constructor(config: AuthConfig, authMachine: AuthMachine, formEntry: AuthFormEntryStore);
    /** {@inheritDoc AuthStore.isPending} */
    get isPending(): boolean;
    /** {@inheritDoc AuthStore.onMachineEvent} */
    onMachineEvent: EventListener;
    /** {@inheritDoc AuthStore.onLogin} */
    onLogin: EventListener;
    /** {@inheritDoc AuthStore.onLogout} */
    onLogout: EventListener;
    /** {@inheritDoc AuthStore.onChangePassword} */
    onChangePassword: EventListener;
    /** {@inheritDoc AuthStore.onForgotPassword} */
    onForgotPassword: EventListener;
    /** {@inheritDoc AuthStore.onResetPassword} */
    onResetPassword: EventListener;
    /** {@inheritDoc AuthStore.metaMessages} */
    get metaMessages(): string[];
    /** {@inheritDoc AuthStore.errorMessages} */
    get errorMessages(): string[];
}
/**
 * AuthStore DI key.
 * @internal
 */
export declare const AuthStore: import("@microsoft/fast-foundation").InterfaceSymbol<AuthStore>;
/**
 * Get AuthStore from the DI container.
 *
 * @remarks
 * A utility method for host applications that are not using decorators or the DI container.
 *
 * @example
 * ```ts
 * import { getAuthStore } from '@genesislcap/foundation-auth/store';
 * ...
 * private authStore = getAuthStore();
 * ```
 *
 * @public
 */
export declare function getAuthStore(): AuthStore;
export {};
//# sourceMappingURL=auth.d.ts.map