import { __awaiter } from "tslib";
import { FASTElement } from '@microsoft/fast-element';
import { DI, Registration } from '@microsoft/fast-foundation';
import { AuthConfig, defaultAuthConfig } from './config';
/**
 * configure.
 *
 * @remarks
 * Configure the auth micro frontend for host app integration.
 *
 * @example General foundation-auth configuration when the route is visited, ie. lazy loading.
 * ```ts
 * title: 'Authenticate',
 * name: 'auth',
 * path: 'auth',
 * element: async () => {
 *   const { configure } = await import('@genesislcap/foundation-auth/config'); // < import from the subpath
 *   return configure({
 *     hostPath: 'auth',
 *     omitRoutes: ['request-account'],
 *     logo: myCompanyLogo,
 *     background: myAuthBG,
 *     postLoginRedirect: () => {
 *       // You could inspect the user here and route them based on various conditions.
 *       Route.path.push('dashboard');
 *     },
 *   });
 * },
 * layout: authLayout,
 * settings: { public: true },
 * childRouters: true,
 * ```
 *
 * @example Providing template options align to your host application.
 * ```ts
 * title: 'Authenticate',
 * name: 'auth',
 * path: 'auth',
 * element: async () => {
 *   const { configure } = await import('@genesislcap/foundation-auth/config'); // < import from the subpath
 *   return configure({
 *     name: `nexus-app-auth`,
 *     // These will be merged with the packages `defaultTemplateOptions`. You will need to have these components registered in your application.
 *     templateOptions: {
 *       button: 'nexus-button',
 *       checkbox: 'nexus-checkbox-next',
 *       provider: 'template',
 *     },
 *   });
 * },
 * ```
 *
 * @param config - A partial AuthConfig.
 * @public
 */
export function configure(config) {
    return __awaiter(this, void 0, void 0, function* () {
        /**
         * Merge the configs
         */
        const value = Object.assign(Object.assign({}, defaultAuthConfig), config);
        if (config.templateOptions) {
            value.templateOptions = Object.assign(Object.assign({}, defaultAuthConfig.templateOptions), config.templateOptions);
        }
        /**
         * Register a new AuthConfig
         */
        DI.getOrCreateDOMContainer().register(Registration.instance(AuthConfig, value));
        /**
         * Lazily reference and define the micro frontend element post config setting.
         */
        const { FoundationAuth, template, styles } = yield import('../main');
        const { name, attributes, shadowOptions, elementOptions } = value;
        return FASTElement.define(FoundationAuth, {
            name,
            template,
            styles,
            attributes,
            shadowOptions,
            elementOptions,
        });
    });
}
