import { __awaiter, __decorate, __param } from "tslib";
import { AbstractMachine, createErrorStateNode, FetchMachine, isDoneInvokeEvent, isErrorEvent, isEventWithParams, isUnauthorizedError, } from '@genesislcap/foundation-state-machine';
import { User } from '@genesislcap/foundation-user';
import { UUID } from '@genesislcap/foundation-utils';
import { DI } from '@microsoft/fast-foundation';
import { and, assign, createMachine, fromPromise, not, or, raise, sendTo } from 'xstate';
import { AuthConfig } from '../config/config';
import { CredentialManager, CredentialType, } from '../credential';
import { MESSAGE_TYPE } from '../dto';
import { LOGIN_DETAILS_URL, LOGIN_URL, LOGOUT_URL, FORGOT_PASSWORD_URL, CHANGE_PASSWORD_URL, RESET_PASSWORD_URL, } from '../env';
import { AuthMessageMapper, createHTTPHeadersFromDTO, isLoginAckEntity, isLoginDetailsAckEntity, isLoginNackEntity, isLogoutAckEntity, isLogoutNackEntity, isNackDTO, } from '../mapper';
import { AuthRouting } from '../routes/routing';
import { logger } from '../utils';
import { setError } from './actions';
import { defaultAuthMachineContext, isLoginParams, isQueryParams } from './types';
/**
 * Default AuthMachine.
 * @public
 */
let DefaultAuthMachine = class DefaultAuthMachine extends AbstractMachine {
    constructor(user, fetchMachine, cm, uuid, messageMapper, config, routing) {
        super();
        this.user = user;
        this.fetchMachine = fetchMachine;
        this.cm = cm;
        this.uuid = uuid;
        this.messageMapper = messageMapper;
        this.config = config;
        this.routing = routing;
        this.onIdleEvents = () => {
            const actions = ['resetContext', 'setParams'];
            return {
                login: {
                    actions,
                    target: '#authMachine.loggingIn',
                },
                ['forgot-password']: {
                    actions,
                    target: '#authMachine.forgotPassword',
                },
                ['change-password']: {
                    actions,
                    target: '#authMachine.changePassword',
                },
                ['reset-password']: {
                    actions,
                    target: '#authMachine.resetPassword',
                },
            };
        };
        this.dtoTransition = () => ({
            actions: ['setDTO'],
            target: 'mapEntity',
        });
        this.machine = createMachine({
            id: 'authMachine',
            context: ({ input }) => (Object.assign({}, input)),
            initial: 'initializing',
            states: {
                /**
                 * Initializing, programmatically logging in where possible
                 */
                initializing: {
                    tags: ['initializing', 'pending'],
                    meta: {
                        content: 'Initializing, please wait.',
                    },
                    always: [
                        {
                            /**
                             * Handle SSO popups first
                             */
                            guard: and(['hasSSOToken', 'hasWindowOpener']),
                            actions: 'handSSOTokenToWindowOpener',
                            target: '#authMachine.idle',
                        },
                        {
                            /**
                             * Create login params from query params when detected
                             */
                            guard: and(['hasQueryParams', not('hasLoginParams')]),
                            actions: 'setLoginParamsFromQueryParams',
                        },
                        {
                            /**
                             * Params based login (trumps silent)
                             */
                            guard: 'hasLoginParams',
                            actions: raise(({ context }) => ({
                                type: 'login',
                                params: context.params,
                            })),
                            target: '#authMachine.idle',
                        },
                        {
                            /**
                             * Silent login (cookie based)
                             */
                            guard: and([not('hasQueryParams'), not('hasLoginParams')]),
                            target: '#authMachine.silentLogin',
                        },
                    ],
                },
                /**
                 * Idle, waiting for events
                 */
                idle: {
                    tags: ['idle'],
                    on: this.onIdleEvents(),
                },
                /**
                 * Attempt silent login via cookies (details endpoint returns the same user information as login when authorised)
                 */
                silentLogin: {
                    tags: ['silentLogin', 'pending'],
                    meta: {
                        content: 'Attempting silent login, please wait.',
                    },
                    invoke: {
                        id: 'loginDetailsFetcher',
                        src: fetchMachine.machine,
                        onDone: this.dtoTransition(),
                        onError: [
                            {
                                guard: 'isUnauthorizedError',
                                target: 'loggedOut',
                            },
                            {
                                target: 'error.silentLogin',
                            },
                        ],
                    },
                    entry: [
                        sendTo('loginDetailsFetcher', this.createFetchMachineEvent(LOGIN_DETAILS_URL, {
                            DETAILS: {},
                        })),
                    ],
                },
                /**
                 * Login with user credentials or tokens
                 */
                loggingIn: {
                    tags: ['loggingIn', 'pending'],
                    meta: {
                        content: 'Logging in, please wait.',
                    },
                    invoke: {
                        id: 'loginFetcher',
                        src: fetchMachine.machine,
                        onDone: this.dtoTransition(),
                        onError: 'error.login',
                    },
                    entry: [
                        sendTo('loginFetcher', ({ context }) => {
                            const params = context.params;
                            const mapper = this.messageMapper.get(MESSAGE_TYPE.LOGIN);
                            const message = mapper.toDTO(params);
                            return this.createFetchMachineEvent(LOGIN_URL, message);
                        }),
                    ],
                },
                loggedIn: {
                    tags: ['loggedIn'],
                    meta: {
                        content: 'Logged in.',
                    },
                    entry: ['storeBasicCredentials', 'loggedInCallback', 'postLoginRedirect'],
                    on: {
                        logout: {
                            target: '#authMachine.loggingOut',
                        },
                    },
                },
                /**
                 * Logout
                 */
                loggingOut: {
                    tags: ['loggingOut', 'pending'],
                    meta: {
                        content: 'Logging out, please wait.',
                    },
                    invoke: {
                        id: 'logoutFetcher',
                        src: fetchMachine.machine,
                        onDone: this.dtoTransition(),
                        onError: 'error.logout',
                    },
                    entry: [
                        sendTo('logoutFetcher', this.createFetchMachineEvent(LOGOUT_URL), {
                            delay: 'logoutFeedbackDelay',
                        }),
                    ],
                },
                loggedOut: {
                    tags: ['loggedOut'],
                    entry: [
                        'unsetUser',
                        'preventSilentAccess',
                        'loggedOutCallback',
                        'postLogoutRedirect',
                        'openOIDCLogoutPage',
                    ],
                    on: this.onIdleEvents(),
                },
                /**
                 * Self-service flows
                 */
                forgotPassword: {
                    tags: ['forgotPassword', 'pending'],
                    meta: {
                        content: 'Sending reset instructions, please wait.',
                    },
                    invoke: {
                        id: 'forgotPasswordFetcher',
                        src: fetchMachine.machine,
                        onDone: 'feedback.forgotPassword',
                        onError: 'error.forgotPassword',
                    },
                    entry: [
                        sendTo('forgotPasswordFetcher', ({ context }) => {
                            const params = context.params;
                            const mapper = this.messageMapper.get(MESSAGE_TYPE.FORGOT_PASSWORD);
                            const message = mapper.toDTO(params);
                            return this.createFetchMachineEvent(FORGOT_PASSWORD_URL, message);
                        }),
                    ],
                },
                changePassword: {
                    tags: ['changePassword', 'pending'],
                    meta: {
                        content: 'Changing password, please wait.',
                    },
                    invoke: {
                        id: 'changePasswordFetcher',
                        src: fetchMachine.machine,
                        onDone: [
                            {
                                guard: 'is200NackDTO',
                                target: 'error.changePassword',
                            },
                            {
                                actions: [
                                    ({ context }) => {
                                        const params = context.params;
                                        this.cm.storeCredentials({
                                            id: params.username,
                                            name: params.username,
                                            password: params.newPassword,
                                        });
                                    },
                                ],
                                target: 'feedback.changePassword',
                            },
                        ],
                        onError: 'error.changePassword',
                    },
                    entry: [
                        sendTo('changePasswordFetcher', ({ context }) => {
                            const params = context.params;
                            const mapper = this.messageMapper.get(MESSAGE_TYPE.CHANGE_PASSWORD);
                            const message = mapper.toDTO(params);
                            return this.createFetchMachineEvent(CHANGE_PASSWORD_URL, message);
                        }),
                    ],
                },
                resetPassword: {
                    tags: ['resetPassword', 'pending'],
                    meta: {
                        content: 'Resetting password, please wait.',
                    },
                    invoke: {
                        id: 'resetPasswordFetcher',
                        src: fetchMachine.machine,
                        onDone: {
                            actions: [
                                ({ context }) => {
                                    const params = context.params;
                                    this.cm.storeCredentials({
                                        id: params.username,
                                        name: params.username,
                                        password: params.newPassword,
                                    });
                                },
                            ],
                            target: 'feedback.resetPassword',
                        },
                        onError: 'error.resetPassword',
                    },
                    entry: [
                        sendTo('resetPasswordFetcher', ({ context }) => {
                            const params = context.params;
                            const mapper = this.messageMapper.get(MESSAGE_TYPE.RESET_PASSWORD);
                            const message = mapper.toDTO(params);
                            return this.createFetchMachineEvent(RESET_PASSWORD_URL, message);
                        }),
                    ],
                },
                /**
                 * Map the server responses to entities
                 */
                mapEntity: {
                    tags: ['mapEntity', 'pending'],
                    meta: {
                        content: 'Mapping entity, please wait.',
                    },
                    entry: 'unsetEntity',
                    invoke: {
                        src: 'mapper',
                        input: ({ context }) => {
                            const mapper = this.messageMapper.get(context.dto.MESSAGE_TYPE);
                            if (!mapper) {
                                throw new Error(`No message mapper assigned for ${context.dto.MESSAGE_TYPE}`);
                            }
                            return {
                                dto: context.dto,
                                mapper: mapper.toEntity,
                            };
                        },
                        onDone: {
                            actions: ['setEntity'],
                            target: '#authMachine.entityCheck',
                        },
                        onError: '#authMachine.error.mapping',
                    },
                },
                /**
                 * Check entity types and drive next step
                 */
                entityCheck: {
                    tags: ['entityCheck', 'pending'],
                    meta: {
                        content: 'Entity check, please wait.',
                    },
                    always: [
                        {
                            guard: or(['isLoginAckEntity', 'isLoginDetailsAckEntity']),
                            actions: ['setUser'],
                            target: '#authMachine.authCheck',
                        },
                        {
                            guard: and(['isLoginNackEntity', 'hasIncorrectCredentials']),
                            target: '#authMachine.feedback.incorrectCredentials',
                        },
                        {
                            guard: and(['isLoginNackEntity', 'isMFARequired']),
                            actions: 'storeBasicCredentials',
                            target: '#authMachine.feedback.mfaRequired',
                        },
                        {
                            guard: and(['isLoginNackEntity', 'hasPasswordExpired']),
                            actions: 'storeBasicCredentials',
                            target: '#authMachine.feedback.passwordExpired',
                        },
                        {
                            guard: 'isLoginNackEntity',
                            target: '#authMachine.error.login',
                        },
                        {
                            guard: 'isLogoutAckEntity',
                            actions: ['unsetUser'],
                            target: '#authMachine.authCheck',
                        },
                        {
                            guard: 'isLogoutNackEntity',
                            target: '#authMachine.error.logout',
                        },
                        {
                            target: '#authMachine.error.mapping',
                        },
                    ],
                },
                /**
                 * Check authentication status
                 */
                authCheck: {
                    tags: ['authCheck', 'pending'],
                    meta: {
                        content: 'Authentication check, please wait.',
                    },
                    always: [
                        {
                            guard: and(['isAuthenticated', not('isBanned')]),
                            target: 'loggedIn',
                        },
                        {
                            target: 'loggedOut',
                        },
                    ],
                },
                /**
                 * Feedback states. These are transient and revert back to idle after a short delay.
                 * These can help improve UX in when the backend nack text is lacking clarity, as with incorrect credentials.
                 */
                feedback: {
                    initial: 'none',
                    states: {
                        none: {
                            tags: ['feedback', 'none'],
                        },
                        changePassword: {
                            tags: ['feedback', 'changePassword'],
                            meta: {
                                content: 'MESSAGE_PASSWORD_CHANGE_SUCCESS',
                            },
                            after: {
                                feedbackDelay: '#authMachine.idle',
                            },
                        },
                        forgotPassword: {
                            tags: ['feedback', 'forgotPassword'],
                            meta: {
                                content: 'MESSAGE_FORGOT_PASSWORD_RESET',
                            },
                            after: {
                                feedbackDelay: '#authMachine.idle',
                            },
                        },
                        resetPassword: {
                            tags: ['feedback', 'resetPassword'],
                            meta: {
                                content: 'MESSAGE_PASSWORD_RESET_SUCCESS',
                            },
                            after: {
                                feedbackDelay: '#authMachine.idle',
                            },
                        },
                        incorrectCredentials: {
                            tags: ['feedback', 'incorrectCredentials'],
                            meta: {
                                content: 'MESSAGE_INCORRECT_CREDENTIALS',
                            },
                            after: {
                                feedbackDelay: '#authMachine.idle',
                            },
                        },
                        mfaRequired: {
                            tags: ['feedback', 'mfaRequired'],
                            meta: {
                                content: 'MESSAGE_MFA_REQUIRED',
                            },
                            after: {
                                feedbackDelay: '#authMachine.idle',
                            },
                        },
                        passwordExpired: {
                            tags: ['feedback', 'passwordExpired'],
                            meta: {
                                content: 'MESSAGE_PASSWORD_EXPIRED',
                            },
                            after: {
                                feedbackDelay: {
                                    actions: 'resetPasswordRedirect',
                                    target: '#authMachine.idle',
                                },
                            },
                        },
                    },
                },
                /**
                 * Error states
                 */
                error: {
                    initial: 'unknown',
                    entry: ['setError', 'errorCallback'],
                    states: {
                        unknown: createErrorStateNode('unknown', 'ERROR_UNKNOWN_ERROR'),
                        login: createErrorStateNode('login', 'ERROR_LOGIN'),
                        silentLogin: createErrorStateNode('silent-login', 'ERROR_SILENT_LOGIN'),
                        changePassword: createErrorStateNode('change-password', 'ERROR_CHANGE_PASSWORD'),
                        forgotPassword: createErrorStateNode('forgot-password', 'ERROR_FORGOT_PASSWORD'),
                        resetPassword: createErrorStateNode('reset-password', 'ERROR_RESET_PASSWORD'),
                        logout: createErrorStateNode('logout', 'ERROR_LOGOUT_ERROR'),
                        mapping: createErrorStateNode('mapping', 'ERROR_MAPPING'),
                    },
                    on: this.onIdleEvents(),
                },
            },
            on: {
                '*': {
                    actions: ({ event }) => {
                        logger.debug('authMachine: unexpected event', event);
                    },
                },
            },
        }, {
            actors: {
                mapper: fromPromise(({ input }) => __awaiter(this, void 0, void 0, function* () { return input.mapper(input.dto); })),
            },
            actions: {
                setParams: assign(({ event }) => {
                    if (isEventWithParams(event)) {
                        return {
                            params: event.params,
                        };
                    }
                }),
                setLoginParamsFromQueryParams: assign({
                    params: ({ context }) => this.convertQueryParamsToLoginParams(context.params),
                }),
                setDTO: assign(({ event }) => {
                    if (isDoneInvokeEvent(event)) {
                        return {
                            dto: event.output.data,
                        };
                    }
                }),
                setEntity: assign(({ event }) => {
                    if (isDoneInvokeEvent(event)) {
                        return {
                            entity: event.output,
                        };
                    }
                }),
                unsetEntity: assign({
                    entity: () => undefined,
                }),
                setUser: ({ event }) => {
                    if (isDoneInvokeEvent(event)) {
                        this.user.set(event.output);
                    }
                },
                unsetUser: () => this.user.unset(),
                handSSOTokenToWindowOpener: ({ context }) => {
                    if (window.opener && isQueryParams(context.params)) {
                        window.opener.sessionStorage.setItem('ssoToken', context.params.SSO_TOKEN);
                        window.opener.location.reload();
                        window.close();
                    }
                },
                openOIDCLogoutPage: ({ event }) => {
                    if (isDoneInvokeEvent(event) && event.output.oidcLogoutUrl) {
                        window.open(event.output.oidcLogoutUrl, '_self');
                    }
                },
                setError,
                resetContext: assign(() => this.startingContext),
                preventSilentAccess: () => __awaiter(this, void 0, void 0, function* () {
                    yield this.cm.preventSilentAccess();
                }),
                storeBasicCredentials: ({ context }) => {
                    const loginParams = context.params;
                    if ((loginParams === null || loginParams === void 0 ? void 0 : loginParams.type) !== CredentialType.BASIC) {
                        return;
                    }
                    const data = {
                        id: loginParams.username,
                        name: loginParams.username,
                        password: loginParams.password,
                    };
                    if (this.cm.validateUserCredentialData(data, true)) {
                        this.cm.storeCredentials(data);
                    }
                },
                postLoginRedirect: this.config.postLoginRedirect,
                postLogoutRedirect: this.config.postLogoutRedirect,
                resetPasswordRedirect: () => this.routing.navigateTo('reset-password?expired'),
                loggedInCallback: () => logger.debug('AuthMachine.loggedInCallback invoked.'),
                loggedOutCallback: () => logger.debug('AuthMachine.loggedOutCallback invoked.'),
                errorCallback: () => logger.debug('AuthMachine.errorCallback invoked.'),
            },
            guards: {
                hasParams: ({ context }) => context.params !== undefined,
                hasLoginParams: ({ context }) => isLoginParams(context.params),
                hasQueryParams: ({ context }) => isQueryParams(context.params),
                hasSSOToken: ({ context }) => isQueryParams(context.params) && context.params.SSO_TOKEN !== undefined,
                hasMFACode: ({ context }) => isQueryParams(context.params) && context.params.MFA_CODE !== undefined,
                hasWindowOpener: () => window.opener,
                isUnauthorizedError: ({ event }) => isErrorEvent(event) ? isUnauthorizedError(event.data) : false,
                isAuthenticated: ({ context }) => { var _a; return (_a = context.user) === null || _a === void 0 ? void 0 : _a.isAuthenticated; },
                isBanned: ({ context }) => { var _a; return (_a = context.user) === null || _a === void 0 ? void 0 : _a.isBanned(); },
                isLoginAckEntity: ({ event }) => isDoneInvokeEvent(event) ? isLoginAckEntity(event.output) : false,
                isLoginDetailsAckEntity: ({ event }) => isDoneInvokeEvent(event) ? isLoginDetailsAckEntity(event.output) : false,
                isLoginNackEntity: ({ event }) => isDoneInvokeEvent(event) ? isLoginNackEntity(event.output) : false,
                hasIncorrectCredentials: ({ event }) => isDoneInvokeEvent(event)
                    ? isLoginNackEntity(event.output) && event.output.hasIncorrectCredentials
                    : false,
                isMFARequired: ({ event }) => isDoneInvokeEvent(event)
                    ? isLoginNackEntity(event.output) && event.output.isMFARequired
                    : false,
                hasPasswordExpired: ({ event }) => isDoneInvokeEvent(event)
                    ? isLoginNackEntity(event.output) && event.output.hasPasswordExpired
                    : false,
                isLogoutAckEntity: ({ event }) => isDoneInvokeEvent(event) ? isLogoutAckEntity(event.output) : false,
                isLogoutNackEntity: ({ event }) => isDoneInvokeEvent(event) ? isLogoutNackEntity(event.output) : false,
                /**
                 * Required until strict http status codes are enabled on the backend by default.
                 */
                is200NackDTO: ({ event }) => {
                    var _a, _b;
                    return isDoneInvokeEvent(event)
                        ? ((_b = (_a = event.output) === null || _a === void 0 ? void 0 : _a.response) === null || _b === void 0 ? void 0 : _b.ok) && isNackDTO(event.output.data)
                        : false;
                },
            },
            delays: {
                feedbackDelay: this.config.feedbackDelay,
                logoutFeedbackDelay: 1000,
            },
        });
        this.start();
    }
    start(options = {
        input: Object.assign(Object.assign({}, defaultAuthMachineContext), { user: this.user, params: this.captureAuthQueryParams() }),
    }) {
        super.start(options);
    }
    /**
     * AuthMachine is a singleton.
     */
    getFromContainer() {
        return this.container.get(AuthMachine);
    }
    captureAuthQueryParams() {
        const queryParams = new URLSearchParams(window.location.search);
        return {
            SSO_TOKEN: queryParams.get('SSO_TOKEN'),
            MFA_CODE: queryParams.get('MFA_CODE'),
        };
    }
    convertQueryParamsToLoginParams(params) {
        const ssoCredentials = params.SSO_TOKEN
            ? {
                type: CredentialType.SSO,
                ssoToken: params.SSO_TOKEN,
            }
            : undefined;
        if (this.cm.validateSSOCredentials(ssoCredentials)) {
            logger.debug(`SSO credentials detected. `);
            return ssoCredentials;
        }
        const mfaCredentials = params.MFA_CODE
            ? {
                type: CredentialType.MFA,
                mfaToken: params.MFA_CODE,
            }
            : undefined;
        if (this.cm.validateMFACredentials(mfaCredentials)) {
            logger.debug(`MFA credentials detected.`);
            return mfaCredentials;
        }
    }
    createFetchMachineEvent(url, msg = {}) {
        const message = Object.assign({ SOURCE_REF: this.uuid.createId() }, msg);
        return {
            type: 'fetch',
            params: {
                url,
                requestInit: {
                    method: 'POST',
                    body: JSON.stringify(message),
                    headers: createHTTPHeadersFromDTO(message),
                    credentials: 'include',
                },
            },
        };
    }
};
DefaultAuthMachine = __decorate([
    __param(0, User),
    __param(1, FetchMachine),
    __param(2, CredentialManager),
    __param(3, UUID),
    __param(4, AuthMessageMapper),
    __param(5, AuthConfig),
    __param(6, AuthRouting)
], DefaultAuthMachine);
export { DefaultAuthMachine };
/**
 * AuthMachine DI key.
 * @internal
 */
export const AuthMachine = DI.createInterface((x) => x.singleton(DefaultAuthMachine));
/**
 * Get AuthMachine from the DI container.
 *
 * @remarks
 * A utility method for host applications that are not using decorators or the DI container.
 *
 * @example
 * ```ts
 * import { getAuthMachine } from '@genesislcap/foundation-auth/machine';
 * import { getUser, UserAuthenticationChangeEvent } from '@genesislcap/foundation-user';
 * ...
 * private authMachine = getAuthMachine();
 * ...
 * private user = getUser();
 * private unsubscribeUser = this.user.addAuthenticationChangeListener((event: UserAuthenticationChangeEvent) => {
 *   console.log('User authenticated?', event.detail); // < boolean === this.user.isAuthenticated
 * });
 * ...
 * this.authMachine.send({
 *   type: 'login',
 *   params: {
 *     type: CredentialType.BASIC,
 *     username: 'JohnWick',
 *     password: '***********',
 *   },
 * });
 * ```
 *
 * @public
 */
export function getAuthMachine() {
    return DI.getOrCreateDOMContainer().get(AuthMachine);
}
