import { __awaiter, __decorate } from "tslib";
import { EventEmitter } from '@genesislcap/foundation-events';
import { customElement, FASTElement, observable, volatile, } from '@microsoft/fast-element';
import { Container, inject, Registration } from '@microsoft/fast-foundation';
import { DefaultRouteRecognizer } from '@microsoft/fast-router';
import { classNames } from '@microsoft/fast-web-utilities';
import '../components/core-components';
import { AuthRouterConfig } from '../routes/config';
import { AuthStore } from '../store';
import { styles } from './main.styles';
import { template } from './main.template';
/**
 * FoundationAuth.
 *
 * @remarks
 * Base MF export used by host application `configure` calls. It does not set up a design system or components. Both are
 * expected to be registered in the host application, which should provide the details of which to the MF via
 * TemplateOptions.
 *
 * See the {@link configure} hook for more information on how to set up and use this micro frontend in your application.
 *
 * @fires store-connected - Fires a custom 'store-connected' event after connection to connect the store.
 * @fires store-ready - Fires a custom 'store-ready' event after connection to ready the store.
 * @fires store-disconnected - Fires a custom 'store-disconnected' event after disconnection to unbind the store.
 *
 * @public
 */
export class FoundationAuth extends EventEmitter(FASTElement) {
    /**
     * @internal
     */
    connectedCallback() {
        this.registerDIDependencies();
        super.connectedCallback();
        this.addBindings();
        this.addEventListeners();
        this.readyStore();
    }
    /**
     * @internal
     */
    disconnectedCallback() {
        super.disconnectedCallback();
        this.removeBindings();
        this.removeEventListeners();
        this.disconnectStore();
    }
    /**
     * @internal
     */
    registerDIDependencies() {
        this.container.register(Registration.transient(DefaultRouteRecognizer, DefaultRouteRecognizer));
    }
    /**
     * @internal
     */
    addBindings() {
        this.machineBinding = this.store.authMachine.binding((x) => x.snapshot.value, (value) => {
            if (value === 'loggedOut') {
                this.attemptAutofill();
            }
        });
    }
    /**
     * @internal
     */
    removeBindings() {
        this.machineBinding.disconnect();
    }
    /**
     * @internal
     */
    addEventListeners() {
        this.addEventListener('store-connected', this.store.onConnected);
    }
    /**
     * @internal
     */
    removeEventListeners() {
        this.removeEventListener('store-connected', this.store.onConnected);
    }
    /**
     * @internal
     */
    readyStore() {
        this.$emit('store-connected', this);
        this.$emit('store-ready', true);
    }
    /**
     * @internal
     */
    disconnectStore() {
        this.$emit('store-disconnected');
    }
    /**
     * @internal
     */
    get isReady() {
        return (!this.store.authMachine.hasTag('initializing') &&
            !this.store.authMachine.hasTag('silentLogin'));
    }
    /**
     * @internal
     */
    get classNames() {
        return classNames(['ready', this.isReady], ['not-ready', !this.isReady]);
    }
    /**
     * @internal
     */
    attemptAutofill() {
        const { autofilling, ssoToggled, username, password } = this.store.formEntry;
        if (!autofilling && !ssoToggled && !username && !password) {
            this.$emit('auth-autofill-form');
        }
    }
}
__decorate([
    AuthStore
], FoundationAuth.prototype, "store", void 0);
__decorate([
    inject(AuthRouterConfig)
], FoundationAuth.prototype, "config", void 0);
__decorate([
    Container
], FoundationAuth.prototype, "container", void 0);
__decorate([
    volatile
], FoundationAuth.prototype, "classNames", null);
/**
 * ZeroAuth.
 *
 * @remarks
 * A zero version that pre-registers zero components and uses the zero design system so that it can run standalone.
 * Generally this is used for development purposes, but can be deployed as a standalone application if required or be
 * used by a host application that is already using zero. ZeroAuth and its components won't bloat the core bundle as
 * these are only loaded on-demand when in use.
 *
 * @example
 * ```ts
 * import { ZeroAuth } from '@genesislcap/foundation-auth';
 * ...
 * title: 'Authenticate',
 * name: 'auth',
 * path: '',
 * element: ZeroAuth,
 * ```
 *
 * @example Load the micro frontend on-demand
 * ```ts
 * title: 'Authenticate',
 * name: 'auth',
 * path: '',
 * element: async () => {
 *   const { ZeroAuth } = await import('@genesislcap/foundation-auth');
 *   return ZeroAuth;
 * },
 * ```
 *
 * @public
 */
let ZeroAuth = class ZeroAuth extends FoundationAuth {
    constructor() {
        super(...arguments);
        /**
         * @internal
         */
        this.zeroReady = false;
    }
    /**
     * @internal
     */
    connectedCallback() {
        const _super = Object.create(null, {
            connectedCallback: { get: () => super.connectedCallback }
        });
        return __awaiter(this, void 0, void 0, function* () {
            _super.connectedCallback.call(this);
            yield this.loadRemotes();
        });
    }
    /**
     * @internal
     */
    loadRemotes() {
        return __awaiter(this, void 0, void 0, function* () {
            const { registerCommonZeroComponents } = yield import('../components/zero-components');
            yield registerCommonZeroComponents();
            this.zeroReady = true;
        });
    }
    /**
     * @internal
     * @privateRemarks
     * The order of the check is important here, check super first.
     */
    get isReady() {
        return super.isReady && this.zeroReady;
    }
};
__decorate([
    observable
], ZeroAuth.prototype, "zeroReady", void 0);
ZeroAuth = __decorate([
    customElement({
        name: 'zero-auth',
        template,
        styles,
    })
], ZeroAuth);
export { ZeroAuth };
