/**
 * @public
 */
export const fromDTOStringArray = (arr) => Array.isArray(arr) ? arr.map((v) => v !== null && v !== void 0 ? v : '') : [];
/**
 * @internal
 */
const fromDTONackErrorWarningArray = (arr) => Array.isArray(arr)
    ? arr.map((v) => ({
        type: v['@type'],
        code: v.CODE,
        statusCode: v.STATUS_CODE,
        text: v.TEXT,
    }))
    : [];
/**
 * includesErrorWarningCode
 * @internal
 */
const includesErrorWarningCode = (arr, code) => Array.isArray(arr) ? arr.some((e) => e.CODE === code || e.STATUS_CODE === code) : false;
/**
 * getNormalisedStatusCode
 * @param arr - A NackErrorWarning array.
 * @returns The highest status code.
 * @internal
 */
const getNormalisedStatusCode = (arr) => {
    if (!Array.isArray(arr) || arr.length === 0) {
        return;
    }
    const codes = arr.map((msg) => {
        try {
            return parseInt(msg.statusCode);
        }
        catch (e) {
            return 0;
        }
    });
    return codes.sort().pop();
};
/**
 * getNormalisedText
 * @param arr - A NackErrorWarning array.
 * @returns A combined text string.
 * @internal
 */
export const getNormalisedText = (arr) => Array.isArray(arr) && arr.length ? arr.map((msg) => msg.text).join(' ') : undefined;
/**
 * hasDTOErrorWarningCode
 * @remarks
 * Checks both the `CODE` and `STATUS_CODE` values for a match.
 * @param dto - NACK.
 * @param code - The error or warning code.
 * @public
 */
export const hasDTOErrorWarningCode = (dto, code) => {
    const errorMatch = includesErrorWarningCode(dto.ERROR, code);
    const warningMatch = includesErrorWarningCode(dto.WARNING, code);
    return errorMatch || warningMatch;
};
/**
 * fromDTOAck
 * @param dto - ACK.
 * @public
 */
export const fromDTOAck = (dto) => ({
    messageType: dto.MESSAGE_TYPE,
    sourceRef: dto.SOURCE_REF,
});
/**
 * fromDTONack
 * @param dto - NACK.
 * @public
 */
export const fromDTONack = (dto) => {
    const error = fromDTONackErrorWarningArray(dto.ERROR);
    const warning = fromDTONackErrorWarningArray(dto.WARNING);
    return {
        messageType: dto.MESSAGE_TYPE,
        sourceRef: dto.SOURCE_REF,
        error,
        warning,
        normalisedErrorStatusCode: getNormalisedStatusCode(error),
        normalisedWarningStatusCode: getNormalisedStatusCode(warning),
        normalisedErrorText: getNormalisedText(error),
        normalisedWarningText: getNormalisedText(warning),
    };
};
/**
 * fromIDPDTO
 * @param dto - IDENTITY_PROVIDER.
 * @public
 */
export const fromIDPDTO = (dto) => {
    return {
        id: dto.ID,
        type: dto.TYPE,
        description: dto.DESCRIPTION,
    };
};
/**
 * isNackDTO.
 * @param dto - Potential NACK.
 * @public
 */
export function isNackDTO(dto) {
    var _a;
    return (_a = dto === null || dto === void 0 ? void 0 : dto.MESSAGE_TYPE) === null || _a === void 0 ? void 0 : _a.endsWith('_NACK');
}
/**
 * isNackEntity.
 * @param entity - Potential Nack.
 * @public
 */
export function isNackEntity(entity) {
    var _a;
    return (_a = entity === null || entity === void 0 ? void 0 : entity.messageType) === null || _a === void 0 ? void 0 : _a.endsWith('_NACK');
}
/**
 * createHTTPHeadersFromDTO.
 * @param dto - DTO being sent to the server.
 * @param contentType - Optional content-type, defaults to `application/json; charset=UTF-8`.
 * @public
 */
export function createHTTPHeadersFromDTO(dto, contentType = 'application/json; charset=UTF-8') {
    const headers = { 'Content-type': contentType };
    if (dto.SOURCE_REF) {
        headers['source-ref'] = dto.SOURCE_REF;
        headers['SOURCE_REF'] = dto.SOURCE_REF;
    }
    if (dto.USER_NAME) {
        headers['user-name'] = dto.USER_NAME;
        headers['USER_NAME'] = dto.USER_NAME;
    }
    return headers;
}
