import { __decorate } from "tslib";
import { createInputEmitter, createSelectEmitter, EventEmitter, targetChecked, } from '@genesislcap/foundation-events';
import { I18next } from '@genesislcap/foundation-i18n';
import { User } from '@genesislcap/foundation-user';
import { toElementStyles } from '@genesislcap/foundation-utils';
import { SecondaryRapidColorHEX } from '@genesislcap/foundation-zero';
import { attr, css, customElement, FASTElement, html, observable, when, } from '@microsoft/fast-element';
import { Container } from '@microsoft/fast-foundation';
import { classNames } from '@microsoft/fast-web-utilities';
import merge from 'lodash/merge';
import { AuthConfig, defaultAuthConfig } from '../config/config';
import { AuthStore } from '../store';
import { buttonTag, checkboxTag, progressRingTag, textFieldTag } from '../tags';
import { AuthRouting } from './routing';
/**
 * @internal
 */
export const inputEmit = createInputEmitter();
/**
 * @internal
 */
export const selectEmit = createSelectEmitter();
/**
 * @internal
 * @privateRemarks
 * For services / constructor injection, using `@optional(AuthConfig) config = defaultAuthConfig;` works as expected.
 * However, for elements, the `@optional` decorator isn't viable. This base element provides an alternative approach.
 */
export class ConfigHostElement extends EventEmitter(FASTElement) {
    connectedCallback() {
        this.config = this.container.has(AuthConfig, true)
            ? this.container.get(AuthConfig)
            : defaultAuthConfig;
        const localizationResources = merge(defaultAuthConfig.localizationResources, this.config.localizationResources);
        this.i18next.addResources(localizationResources);
        super.connectedCallback();
    }
}
__decorate([
    Container
], ConfigHostElement.prototype, "container", void 0);
__decorate([
    observable
], ConfigHostElement.prototype, "config", void 0);
__decorate([
    I18next
], ConfigHostElement.prototype, "i18next", void 0);
/**
 * @internal
 */
export class BaseRoute extends ConfigHostElement {
    constructor() {
        super(...arguments);
        this.onBack = () => this.routing.navigateTo('/');
    }
    toLocalisedText(text) {
        return this.i18next.t(`${defaultAuthConfig.name}:${text}`, { lng: this.i18next.language });
    }
}
__decorate([
    User
], BaseRoute.prototype, "user", void 0);
__decorate([
    AuthStore
], BaseRoute.prototype, "store", void 0);
__decorate([
    AuthRouting
], BaseRoute.prototype, "routing", void 0);
/**
 * @internal
 */
export const organisationField = () => html `
  ${when((x) => x.config.fields.organisation, html `
      <${textFieldTag}
        type="text"
        name="organisation"
        id="organisation"
        autocomplete="organization"
        data-test-id="organisation"
        :value="${(x) => x.store.formEntry.organisation}"
        @input="${inputEmit('auth-organisation-changed')}"
        autofocus
        class="${(x) => (!x.config.logo ? 'no-top-margin' : '')}"
        ?required="${(x) => x.config.fields.organisation.required !== false}"
      >
        <span class="form-title">
          ${(x) => { var _a; return x.toLocalisedText((_a = x.config.fields.organisation.label) !== null && _a !== void 0 ? _a : 'ORGANISATION_TITLE'); }}
        </span>
      </${textFieldTag}>
    `)}
`;
/**
 * @internal
 */
export const usernameField = () => html `
  <${textFieldTag}
    type="text"
    name="username"
    id="username"
    autocomplete="username email"
    data-test-id="username"
    :value="${(x) => x.store.formEntry.username}"
    @input="${inputEmit('auth-username-changed')}"
    ?autofocus="${(x) => !x.config.fields.organisation}"
    class="${(x) => (!x.config.logo && !x.config.fields.organisation ? 'no-top-margin' : '')}"
    pattern="${(x) => x.config.fields.username.pattern}"
    title="${(x) => x.config.fields.username.title}"
    required
  >
    <span class="form-title">${(x) => { var _a; return x.toLocalisedText((_a = x.config.fields.username.label) !== null && _a !== void 0 ? _a : 'FIELD_USERNAME_TITLE'); }}</span>
  </${textFieldTag}>
`;
/**
 * @internal
 */
export const emailField = () => html `
  ${when((x) => x.config.fields.email, html `
      <${textFieldTag}
        type="email"
        name="email"
        id="email"
        autocomplete="email"
        data-test-id="email"
        :value="${(x) => x.store.formEntry.email}"
        @input="${inputEmit('auth-email-changed')}"
        required
      >
        <span class="form-title">${(x) => { var _a; return x.toLocalisedText((_a = x.config.fields.email.label) !== null && _a !== void 0 ? _a : 'FIELD_EMAIL_TITLE'); }}</span>
      </${textFieldTag}>
    `)}
`;
/**
 * @internal
 */
export const passwordField = (label) => html `
  <${textFieldTag}
    type="${(x) => (x.store.formEntry.showPassword ? 'text' : 'password')}"
    name="password"
    id="password"
    autocomplete="current-password"
    data-test-id="password"
    :value="${(x) => x.store.formEntry.password}"
    @input="${inputEmit('auth-password-changed')}"
    required
  >
    <span class="form-title">${(x) => { var _a; return x.toLocalisedText((_a = label !== null && label !== void 0 ? label : x.config.fields.password.label) !== null && _a !== void 0 ? _a : 'FIELD_PASSWORD_TITLE'); }}</span>
  </${textFieldTag}>
`;
const password2FieldMeta = {
    new: ['new-password', 'FIELD_PASSWORD_NEW_TITLE'],
    confirmation: ['password-confirmation', 'FIELD_PASSWORD_CONFIRM_TITLE'],
};
/**
 * @internal
 */
export const password2Field = (type) => {
    const [name, label] = password2FieldMeta[type];
    return html `
    <${textFieldTag}
      type="${(x) => (x.store.formEntry.showPassword ? 'text' : 'password')}"
      name="${name}"
      id="${name}"
      autocomplete="${name}"
      data-test-id="${name}"
      :value="${(x) => x.store.formEntry.password2}"
      @input="${inputEmit('auth-password2-changed')}"
      required
    >
      <span class="form-title">${(x) => x.toLocalisedText(label)}</span>
    </${textFieldTag}>
  `;
};
/**
 * @internal
 * @privateRemarks
 * Temp optimisation. How we handle images like this needs a broader review.
 */
let BackgroundElement = class BackgroundElement extends ConfigHostElement {
    connectedCallback() {
        super.connectedCallback();
        this.backgroundStyles = toElementStyles(this.config.background);
        if (this.backgroundStyles) {
            this.$fastController.addStyles(this.backgroundStyles);
        }
        /**
         * If there's a background video we'll need to ensure there's no image set on the background that blocks it.
         */
        if (this.config.backgroundVideo) {
            this.$fastController.addStyles(css `
        :host {
          background-image: none;
        }
      `);
        }
    }
};
__decorate([
    observable
], BackgroundElement.prototype, "backgroundStyles", void 0);
BackgroundElement = __decorate([
    customElement({
        name: 'auth-background',
        template: html `
    <template class="background" part="background" data-test-id="background">
      <slot></slot>
      ${when((x) => x.config.backgroundVideo, html `
          <video autoplay loop playsinline muted>
            <source
              src="${(x) => x.config.backgroundVideo.src}"
              type="${(x) => x.config.backgroundVideo.type}"
            />
            Your browser does not support HTML5 video.
          </video>
        `)}
    </template>
  `,
        styles: css `
    :host {
      background-color: var(--foundation-auth-background-color, transparent);
      background-position: center;
      background-repeat: no-repeat;
      background-attachment: fixed;
      background-size: cover;
      background-origin: border-box;
      display: flex;
      flex-direction: column;
      align-items: center;
      width: 100%;
      height: 100%;
      overflow: auto;
    }
    ::slotted(div.container) {
      margin: auto;
    }
    video {
      object-fit: cover;
      position: fixed;
      width: 100%;
      height: 100%;
      z-index: -101;
    }
  `,
    })
], BackgroundElement);
/**
 * @internal
 */
let LogoElement = class LogoElement extends ConfigHostElement {
    connectedCallback() {
        super.connectedCallback();
        this.logoStyles = toElementStyles(this.config.logo);
    }
};
__decorate([
    observable
], LogoElement.prototype, "logoStyles", void 0);
LogoElement = __decorate([
    customElement({
        name: 'auth-logo',
        template: html `
    ${when((x) => x.logoStyles, html `
        <img class="logo" style="${(x) => x.logoStyles.styles}" part="logo" data-test-id="logo" />
      `)}
  `,
        styles: css `
    .logo {
      display: initial;
      margin-bottom: calc(var(--design-unit) * 5px);
      transition: opacity 5s ease-in-out;
      width: auto;
      height: auto;
    }
  `,
    })
], LogoElement);
/**
 * @internal
 */
export const backButton = (binding, label) => html `
  <${buttonTag}
    appearance="neutral-grey"
    type="button"
    class="action-btn"
    @click=${binding}
    data-test-id="back"
  >
    ${label}
  </${buttonTag}>
`;
/**
 * @internal
 */
export const submitButton = (label, dataTestId = 'submit', disabled = false) => html `
  <${buttonTag}
    appearance="primary-gradient"
    type="submit"
    class="action-btn"
    data-test-id="${dataTestId}"
    ?disabled="${() => disabled}"
  >
    ${label}
  </${buttonTag}>
`;
/**
 * @internal
 */
export const showPasswordCheckbox = () => html `
  <${checkboxTag}
    ?checked="${(x) => x.store.formEntry.showPassword}"
r   @change="${(x, c) => x.$emit('auth-showPassword-changed', targetChecked(c))}"
    name="show-password"
    data-test-id="show-password"
  >
    Show Password
  </${checkboxTag}>
`;
/**
 * @internal
 * @privateRemarks
 * Temp optimisation. How we handle images like this needs a broader review.
 */
let MessageElement = class MessageElement extends FASTElement {
    get message() {
        const translateHelper = (metaMessage) => this.i18next.t(`${defaultAuthConfig.name}:${metaMessage}`, { lng: this.i18next.language });
        const meta = this.store.metaMessages.map(translateHelper).join('\n');
        const error = this.store.errorMessages.map(translateHelper).join('\n');
        return !error ? meta : `${meta} ${error}.`;
    }
    get hasError() {
        return this.store.errorMessages.length > 0;
    }
    get classNames() {
        return classNames('message', ['has-error', this.hasError]);
    }
};
__decorate([
    AuthStore
], MessageElement.prototype, "store", void 0);
__decorate([
    I18next
], MessageElement.prototype, "i18next", void 0);
MessageElement = __decorate([
    customElement({
        name: 'auth-message',
        template: html `
    ${when((x) => x.message, html `
        <span class="${(x) => x.classNames}" part="message" data-test-id="message">
          ${(x) => x.message}
        </span>
      `)}
  `,
        styles: css `
    :host {
      text-align: center;
    }
    .message {
      color: var(--accent-foreground-rest);
      margin-top: calc(var(--design-unit) * 3px);
      white-space: pre-line;
    }
    .message.has-error {
      color: ${SecondaryRapidColorHEX.orange};
    }
  `,
    })
], MessageElement);
/**
 * @internal
 */
let SubmittingIndicator = class SubmittingIndicator extends FASTElement {
    constructor() {
        super(...arguments);
        this.isSubmitting = false;
    }
};
__decorate([
    attr({ mode: 'boolean', attribute: 'is-submitting' })
], SubmittingIndicator.prototype, "isSubmitting", void 0);
SubmittingIndicator = __decorate([
    customElement({
        name: 'submitting-indicator',
        template: html `
    ${when((x) => x.isSubmitting, html `
        <${progressRingTag}
          part="submitting-indicator"
          data-test-id="submitting-indicator"
        ></${progressRingTag}>
      `)}
  `,
        styles: css `
    :host {
      display: flex;
      justify-content: center;
    }
    ${progressRingTag} {
      align-self: center;
    }
  `,
    })
], SubmittingIndicator);
