import { __awaiter, __decorate, __param } from "tslib";
import { User } from '@genesislcap/foundation-user';
import { Container } from '@microsoft/fast-foundation';
import { RouterConfiguration, Route } from '@microsoft/fast-router';
import { AuthConfig } from '../config/config';
import { defaultLayout } from '../layouts';
import { logger } from '../utils';
import { ForgotPassword } from './forgot-password/forgot-password';
import { Login } from './login/login';
import { Logout } from './logout/logout';
import { NotFound } from './not-found/not-found';
import { RequestAccount } from './request-account/request-account';
import { ResetPassword } from './reset-password/reset-password';
import { AuthRouting } from './routing';
import { Verify } from './verify/verify';
/**
 * Base implementation of a routing used with the Auth class
 *
 * @internal
 */
let AuthRouterConfig = class AuthRouterConfig extends RouterConfiguration {
    constructor(user, container, authConfig, authRouting) {
        super();
        this.user = user;
        this.container = container;
        this.authConfig = authConfig;
        this.authRouting = authRouting;
    }
    configure() {
        this.title = 'Auth';
        this.defaultLayout = defaultLayout;
        const getRoutePath = (route) => this.parent ? route : this.authRouting.getRoutePath(route);
        const routes = [
            {
                path: getRoutePath(''),
                name: 'login',
                element: Login,
                title: 'Login',
                settings: { public: true },
            },
            {
                path: getRoutePath('logout'),
                name: 'logout',
                element: Logout,
                title: 'Logging out, please wait...',
                settings: { public: true },
            },
            {
                path: getRoutePath('not-found'),
                name: 'not-found',
                element: NotFound,
                title: 'Not Found',
                settings: { public: true },
            },
        ];
        const { omitRoutes } = this.authConfig;
        if (!omitRoutes.includes('request-account')) {
            routes.push({
                path: getRoutePath('request-account'),
                name: 'request-account',
                element: RequestAccount,
                title: 'Request Account',
                settings: { public: true },
            });
        }
        if (!omitRoutes.includes('reset-password')) {
            routes.push({
                path: getRoutePath('reset-password'),
                name: 'reset-password',
                element: ResetPassword,
                title: 'Reset Password',
                settings: { public: true },
            });
        }
        if (!omitRoutes.includes('forgot-password')) {
            routes.push({
                path: getRoutePath('forgot-password'),
                name: 'forgot-password',
                element: ForgotPassword,
                title: 'Forgot Password',
                settings: { public: true },
            });
        }
        if (!omitRoutes.includes('verify')) {
            routes.push({
                path: getRoutePath('verify'),
                name: 'verify',
                element: Verify,
                title: 'Verify',
                settings: { public: true },
            });
        }
        if (!omitRoutes.includes('protected')) {
            routes.push({
                path: getRoutePath('protected'),
                name: 'protected',
                element: () => __awaiter(this, void 0, void 0, function* () {
                    const { Protected } = yield import('./protected/protected');
                    return Protected;
                }),
                title: 'Protected',
                settings: { public: false },
            });
        }
        this.routes.map(...routes);
        /**
         * Example of a FallbackRouteDefinition
         */
        this.routes.fallback(() => {
            this.user.trackPath();
            const result = this.user.isAuthenticated
                ? { redirect: getRoutePath('not-found') }
                : { redirect: getRoutePath('') };
            logger.debug('AuthRouterConfig.routes.fallback. Falling back to:', result);
            return result;
        });
        /**
         * Example of a NavigationContributor
         */
        this.contributors.push({
            navigate: (phase) => __awaiter(this, void 0, void 0, function* () {
                const settings = phase.route.settings;
                /**
                 * If public route don't block
                 */
                if (settings && settings.public) {
                    return;
                }
                /**
                 * If authenticated don't block
                 */
                if (this.user.isAuthenticated) {
                    return;
                }
                /**
                 * Otherwise route them to login
                 */
                phase.cancel(() => {
                    this.user.trackPath();
                    Route.name.replace(phase.router, getRoutePath(''));
                });
            }),
        });
    }
    construct(Type) {
        return this.container.get(Type);
    }
};
AuthRouterConfig = __decorate([
    __param(0, User),
    __param(1, Container),
    __param(2, AuthConfig),
    __param(3, AuthRouting)
], AuthRouterConfig);
export { AuthRouterConfig };
