import { __awaiter, __decorate } from "tslib";
import { inIFrame, inSymphonyDesktop, openPopup } from '@genesislcap/foundation-utils';
import { customElement, volatile } from '@microsoft/fast-element';
import { CredentialManager, CredentialType } from '../../credential';
import { logger } from '../../utils';
import { BaseRoute } from '../base';
import { LoginStyles as styles } from './login.styles';
import { LoginTemplate as template } from './login.template';
/**
 * Login
 *
 * @fires auth-login - Fires a custom 'auth-login' event.
 * @fires auth-logout - Fires a custom 'auth-logout' event.
 * @fires auth-ssoToggled-changed - Fires a custom 'auth-ssoToggled-changed' event.
 * @fires auth-selectedIdp-changed - Fires a custom 'auth-selectedIdp-changed' event.
 * @fires auth-organisation-changed - Fires a custom 'auth-organisation-changed' event.
 * @fires auth-username-changed - Fires a custom 'auth-username-changed' event.
 * @fires auth-password-changed - Fires a custom 'auth-password-changed' event.
 * @fires auth-showPassword-changed - Fires a custom 'auth-showPassword-changed' event.
 *
 * @public
 */
let Login = class Login extends BaseRoute {
    connectedCallback() {
        super.connectedCallback();
        /**
         * This is the top-level mf route. If it connects while the user is logged in, we can assume the intention is to
         * logout, and can navigate users to the logout route automatically to simplify host integrations.
         */
        if (this.user.isAuthenticated) {
            this.routing.navigateTo('logout');
            return;
        }
    }
    attemptBasicLogin() {
        const basicCredentials = {
            type: CredentialType.BASIC,
            username: this.store.formEntry.username,
            password: this.store.formEntry.password,
        };
        if (!this.credentialManager.validateBasicCredentials(basicCredentials)) {
            logger.debug(`Basic credentials appear invalid, aborting login attempt.`);
            return;
        }
        logger.debug(`Attempting to login with basic credentials.`);
        this.login(basicCredentials);
    }
    get ssoToggleLabel() {
        const { selectedIdp, ssoToggled } = this.store.formEntry;
        const selectedIDPLabel = selectedIdp ? ` - ${selectedIdp.id}` : '';
        return ssoToggled
            ? this.toLocalisedText('LOGIN_SSO_CHECKBOX_ENABLED_TITLE') + selectedIDPLabel
            : this.toLocalisedText('LOGIN_SSO_CHECKBOX');
    }
    onSSOLogin() {
        const ssoLoginUrl = this.store.formEntry.ssoLoginUrl;
        if (!ssoLoginUrl) {
            logger.error(`SSO login url invalid, aborting login attempt.`);
            return;
        }
        if (inIFrame()) {
            if (inSymphonyDesktop()) {
                // symphony/electron opens new window in the default browser instead
                // popup within electron, so we have to navigate manually
                openPopup('', 'symphony').location.assign(ssoLoginUrl);
            }
            else {
                openPopup(ssoLoginUrl, 'Login');
            }
        }
        else {
            window.open(ssoLoginUrl, '_self');
        }
    }
    onLoginSubmit() {
        return __awaiter(this, void 0, void 0, function* () {
            this.attemptBasicLogin();
        });
    }
    login(params) {
        this.$emit('auth-login', params);
    }
    logout() {
        this.$emit('auth-logout');
    }
    get submitDisabled() {
        return this.store.authMachine.hasTag('pending');
    }
};
__decorate([
    CredentialManager
], Login.prototype, "credentialManager", void 0);
__decorate([
    volatile
], Login.prototype, "ssoToggleLabel", null);
Login = __decorate([
    customElement({
        name: 'foundation-auth-login',
        template,
        styles,
    })
], Login);
export { Login };
