import { __awaiter, __decorate } from "tslib";
import { FetchMachine } from '@genesislcap/foundation-state-machine';
import { DI } from '@microsoft/fast-foundation';
import { SSO_LIST_URL, SSO_LOGIN_URL } from '../env';
import { fromIDPDTO } from '../mapper';
/**
 * Default IDPService.
 * @public
 */
export class DefaultIDPService {
    constructor() {
        /**
         * The pattern matches `{property}` values in the string.
         * @internal
         */
        this.loginUrlReplacePattern = /{\s*(\w+?)\s*}/g;
        /** {@inheritDoc IDPService.getList} */
        this.getList = (url = SSO_LIST_URL) => __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve, reject) => {
                const fetcher = this.fetchMachine.getFromContainer();
                fetcher.provide({
                    actions: {
                        successCallback: ({ context }) => {
                            const { data } = context;
                            /**
                             * Format reference:
                             * const IDPS: IDENTITY_PROVIDER[] = [
                             *   { ID: 'hsbc', DESCRIPTION: 'hsbc', TYPE: 'saml' },
                             *   { ID: 'mufg', DESCRIPTION: 'mufg', TYPE: 'saml' },
                             * ];
                             */
                            const IDPS = data === null || data === void 0 ? void 0 : data.IDPS;
                            if (!IDPS || !IDPS.length) {
                                reject(Error('Identity provider list empty'));
                                return;
                            }
                            const idps = IDPS.map((dto) => fromIDPDTO(dto));
                            resolve(idps);
                        },
                        errorCallback: () => reject(Error('Failed to fetch identity providers')),
                    },
                });
                fetcher.start();
                fetcher.send({
                    type: 'fetch',
                    params: {
                        /**
                         * Used `url: 'https://hsbc-uat-priss1/gwf/sso/list'` during testing.
                         */
                        url,
                    },
                });
            });
        });
        /** {@inheritDoc IDPService.getLoginUrl} */
        this.getLoginUrl = (idp, urlTemplate = SSO_LOGIN_URL) => {
            if (!idp || !idp.type || !idp.id) {
                throw new Error(`Invalid Identity Provider: ${JSON.stringify(idp)}`);
            }
            return urlTemplate.replace(this.loginUrlReplacePattern, (_, token) => idp[token] || '');
        };
    }
}
__decorate([
    FetchMachine
], DefaultIDPService.prototype, "fetchMachine", void 0);
/**
 * IDPService DI key.
 * @internal
 */
export const IDPService = DI.createInterface((x) => x.singleton(DefaultIDPService));
