import { __decorate, __param } from "tslib";
import { AbstractStoreRoot, registerStore, } from '@genesislcap/foundation-store';
import { DI } from '@microsoft/fast-foundation';
import { AuthConfig } from '../config/config';
import { AuthMachine } from '../machine/machine';
import { AuthFormEntryStore } from './authFormEntry';
/**
 * Default AuthStore.
 * @public
 */
let DefaultAuthStore = class DefaultAuthStore extends AbstractStoreRoot {
    constructor(config, authMachine, formEntry) {
        super(formEntry);
        this.config = config;
        this.authMachine = authMachine;
        this.formEntry = formEntry;
        /** {@inheritDoc AuthStore.onMachineEvent} */
        this.onMachineEvent = this.createListener('auth-machine-event', (event) => this.authMachine.send(event));
        /** {@inheritDoc AuthStore.onLogin} */
        this.onLogin = this.createListener('auth-login', (params) => this.authMachine.send({
            type: 'login',
            params,
        }));
        /** {@inheritDoc AuthStore.onLogout} */
        this.onLogout = this.createListener('auth-logout', () => this.authMachine.send({ type: 'logout' }));
        /** {@inheritDoc AuthStore.onChangePassword} */
        this.onChangePassword = this.createListener('auth-change-password', (params) => this.authMachine.send({
            type: 'change-password',
            params,
        }));
        /** {@inheritDoc AuthStore.onForgotPassword} */
        this.onForgotPassword = this.createListener('auth-forgot-password', (params) => this.authMachine.send({
            type: 'forgot-password',
            params,
        }));
        /** {@inheritDoc AuthStore.onResetPassword} */
        this.onResetPassword = this.createListener('auth-reset-password', (params) => this.authMachine.send({
            type: 'reset-password',
            params,
        }));
    }
    /** {@inheritDoc AuthStore.isPending} */
    get isPending() {
        return !this.ready || this.authMachine.hasTag('pending');
    }
    /** {@inheritDoc AuthStore.metaMessages} */
    get metaMessages() {
        return this.authMachine.getMetaValuesByKey();
    }
    /** {@inheritDoc AuthStore.errorMessages} */
    get errorMessages() {
        const messages = [];
        for (const { message } of this.authMachine.errors.values()) {
            messages.push(message);
        }
        for (const { message } of this.formEntry.errors.values()) {
            messages.push(message);
        }
        return messages;
    }
};
DefaultAuthStore = __decorate([
    __param(0, AuthConfig),
    __param(1, AuthMachine),
    __param(2, AuthFormEntryStore)
], DefaultAuthStore);
export { DefaultAuthStore };
/**
 * AuthStore DI key.
 * @internal
 */
export const AuthStore = registerStore(DefaultAuthStore, 'AuthStore');
/**
 * Get AuthStore from the DI container.
 *
 * @remarks
 * A utility method for host applications that are not using decorators or the DI container.
 *
 * @example
 * ```ts
 * import { getAuthStore } from '@genesislcap/foundation-auth/store';
 * ...
 * private authStore = getAuthStore();
 * ```
 *
 * @public
 */
export function getAuthStore() {
    return DI.getOrCreateDOMContainer().get(AuthStore);
}
