import { __awaiter, __decorate } from "tslib";
import { AbstractStore, registerStore } from '@genesislcap/foundation-store';
import { observable, volatile } from '@microsoft/fast-element';
import { AuthConfig } from '../config/config';
import { CredentialManager, credentialSeparator } from '../credential';
import { IDPService } from '../services';
import { logger } from '../utils';
/**
 * Default AuthFormEntryStore.
 * @public
 */
export class DefaultAuthFormEntryStore extends AbstractStore {
    /** {@inheritDoc AuthFormEntryStore.orgUsername} */
    get orgUsername() {
        return this.config.fields.organisation && this.organisation.trim() !== ''
            ? `${this.username.trim()}:${this.organisation.trim()}`
            : this.username.trim();
    }
    constructor() {
        var _a, _b;
        super();
        /** {@inheritDoc AuthFormEntryStore.organisation} */
        this.organisation = '';
        /** {@inheritDoc AuthFormEntryStore.username} */
        this.username = '';
        /** {@inheritDoc AuthFormEntryStore.email} */
        this.email = '';
        /** {@inheritDoc AuthFormEntryStore.password} */
        this.password = '';
        /** {@inheritDoc AuthFormEntryStore.password2} */
        this.password2 = '';
        /** {@inheritDoc AuthFormEntryStore.showPassword} */
        this.showPassword = false;
        /** {@inheritDoc AuthFormEntryStore.hasPasswordExpired} */
        this.hasPasswordExpired = false;
        /** {@inheritDoc AuthFormEntryStore.passwordResetToken} */
        this.passwordResetToken = '';
        /** {@inheritDoc AuthFormEntryStore.mfaCode} */
        this.mfaCode = '';
        /** {@inheritDoc AuthFormEntryStore.idps} */
        this.idps = [];
        /** {@inheritDoc AuthFormEntryStore.ssoToggled} */
        this.ssoToggled = false;
        /** {@inheritDoc AuthFormEntryStore.termsAgreed} */
        this.termsAgreed = false;
        /** {@inheritDoc AuthFormEntryStore.autofilling} */
        this.autofilling = false;
        /** {@inheritDoc AuthFormEntryStore.onAutofillForm} */
        this.onAutofillForm = this.createListener('auth-autofill-form', () => this.autofill());
        /** {@inheritDoc AuthFormEntryStore.onOrganisationChanged} */
        this.onOrganisationChanged = this.createListener('auth-organisation-changed', (detail) => (this.commit.organisation = detail));
        /** {@inheritDoc AuthFormEntryStore.onUsernameChanged} */
        this.onUsernameChanged = this.createListener('auth-username-changed', (detail) => (this.commit.username = detail));
        /** {@inheritDoc AuthFormEntryStore.onEmailChanged} */
        this.onEmailChanged = this.createListener('auth-email-changed', (detail) => (this.commit.email = detail));
        /** {@inheritDoc AuthFormEntryStore.onPasswordChanged} */
        this.onPasswordChanged = this.createListener('auth-password-changed', (detail) => (this.commit.password = detail));
        /** {@inheritDoc AuthFormEntryStore.onPassword2Changed} */
        this.onPassword2Changed = this.createListener('auth-password2-changed', (detail) => (this.commit.password2 = detail));
        /** {@inheritDoc AuthFormEntryStore.onShowPasswordChanged} */
        this.onShowPasswordChanged = this.createListener('auth-showPassword-changed', (detail) => (this.commit.showPassword = detail));
        /** {@inheritDoc AuthFormEntryStore.onSelectedIdpChanged} */
        this.onSelectedIdpChanged = this.createListener('auth-selectedIdp-changed', (detail) => {
            this.commit.selectedIdp = Array.isArray(this.idps)
                ? this.idps.find((idp) => idp.id === detail)
                : undefined;
        });
        /** {@inheritDoc AuthFormEntryStore.onSSOToggleChanged} */
        this.onSSOToggleChanged = this.createListener('auth-ssoToggled-changed', (detail) => (this.commit.ssoToggled = detail));
        /**
         * @privateRemarks
         * Internally observed given constructor assignment.
         * @internal
         */
        this.ssoToggledChanged = (prev, next) => __awaiter(this, void 0, void 0, function* () {
            if (!next) {
                this.resetIDPState();
                this.commit.errors.delete('auth-fetch-idps-error');
                return;
            }
            yield this.invokeAsyncAPI(this.idpService.getList, 'auth-fetch-idps-error', 'auth-fetch-idps-success');
        });
        this.ssoToggled = (_b = (_a = this.config.sso) === null || _a === void 0 ? void 0 : _a.toggled) !== null && _b !== void 0 ? _b : false;
        /**
         * Listeners not on the public interface can be created anonymously
         */
        this.createListener('auth-fetch-idps-success', (idps) => {
            this.commit.idps = idps;
            this.commit.selectedIdp = Array.isArray(idps) ? idps[0] : undefined;
        });
        this.createErrorListener('auth-fetch-idps-error', () => this.resetIDPState());
    }
    /**
     * @internal
     */
    resetIDPState() {
        this.commit.idps = [];
        this.commit.selectedIdp = undefined;
    }
    /**
     * @internal
     */
    autofill() {
        var _a, _b, _c;
        return __awaiter(this, void 0, void 0, function* () {
            let credentialData;
            try {
                /**
                 * Neptune was using cookies to store user credentials. To provide them with a seamless upgrade, we should load
                 * from cookies if the container is empty. These cookies become irrelevant after first login using container.
                 */
                this.autofilling = true;
                const containerData = yield this.credentialManager.getCredentialsFromContainer();
                const cookieData = yield this.credentialManager.getCredentialsFromCookie();
                const [normalisedData, canRemoveCookie] = this.credentialManager.normaliseLegacyCredentialData(containerData, cookieData);
                if (cookieData && canRemoveCookie) {
                    /**
                     * Upgrade path: They have chosen a profile from the container, yet also have a cookie... it's safe to remove the cookie.
                     */
                    this.credentialManager.removeCookie();
                }
                credentialData = normalisedData;
                if (credentialData) {
                    const passwordCredentials = credentialData;
                    this.username = (_a = passwordCredentials.name) !== null && _a !== void 0 ? _a : '';
                    this.password = (_b = passwordCredentials.password) !== null && _b !== void 0 ? _b : '';
                    if (this.config.fields.organisation) {
                        /**
                         * Credential data examples:
                         *
                         * ```
                         * id: JohnDoe:OrgX
                         * password: ************
                         *
                         * id: JJaneDee:OrgY
                         * password: **************
                         *
                         * id: JBob
                         * password: *******
                         * ```
                         *
                         * `valueParts` below is the id split by the `credentialSeparator`:
                         *
                         * ```
                         * ['JohnDoe', 'OrgX'], ['JaneDee', 'OrgY'], ['Bob']
                         * ```
                         */
                        const combinedCompIdUsername = (_c = passwordCredentials.id) !== null && _c !== void 0 ? _c : '';
                        const valueParts = combinedCompIdUsername.split(credentialSeparator);
                        const hasOrg = valueParts.length === 2;
                        const orgIdx = hasOrg ? 1 : 0;
                        const proposedOrganisation = valueParts[orgIdx];
                        if (proposedOrganisation !== this.username) {
                            this.organisation = proposedOrganisation;
                        }
                    }
                }
            }
            catch (err) {
                logger.error('An error occurred attempting to get and autofill from stored credentials.', err);
            }
            this.autofilling = false;
            return Promise.resolve(!!credentialData);
        });
    }
    /** {@inheritDoc AuthFormEntryStore.ssoLoginUrl} */
    get ssoLoginUrl() {
        return this.selectedIdp ? this.idpService.getLoginUrl(this.selectedIdp) : null;
    }
}
__decorate([
    AuthConfig
], DefaultAuthFormEntryStore.prototype, "config", void 0);
__decorate([
    CredentialManager
], DefaultAuthFormEntryStore.prototype, "credentialManager", void 0);
__decorate([
    IDPService
], DefaultAuthFormEntryStore.prototype, "idpService", void 0);
__decorate([
    observable
], DefaultAuthFormEntryStore.prototype, "organisation", void 0);
__decorate([
    observable
], DefaultAuthFormEntryStore.prototype, "username", void 0);
__decorate([
    observable
], DefaultAuthFormEntryStore.prototype, "email", void 0);
__decorate([
    observable
], DefaultAuthFormEntryStore.prototype, "password", void 0);
__decorate([
    observable
], DefaultAuthFormEntryStore.prototype, "password2", void 0);
__decorate([
    observable
], DefaultAuthFormEntryStore.prototype, "showPassword", void 0);
__decorate([
    observable
], DefaultAuthFormEntryStore.prototype, "hasPasswordExpired", void 0);
__decorate([
    observable
], DefaultAuthFormEntryStore.prototype, "passwordResetToken", void 0);
__decorate([
    observable
], DefaultAuthFormEntryStore.prototype, "mfaCode", void 0);
__decorate([
    observable
], DefaultAuthFormEntryStore.prototype, "idps", void 0);
__decorate([
    observable
], DefaultAuthFormEntryStore.prototype, "selectedIdp", void 0);
__decorate([
    observable
], DefaultAuthFormEntryStore.prototype, "ssoToggled", void 0);
__decorate([
    observable
], DefaultAuthFormEntryStore.prototype, "termsAgreed", void 0);
__decorate([
    observable
], DefaultAuthFormEntryStore.prototype, "autofilling", void 0);
__decorate([
    volatile
], DefaultAuthFormEntryStore.prototype, "orgUsername", null);
/**
 * AuthFormEntryStore DI key.
 * @internal
 */
export const AuthFormEntryStore = registerStore(DefaultAuthFormEntryStore, 'AuthFormEntryStore');
